package com.FranckBarbier.Java.Currency_conversion;

/**
 * Simple Java program to test the currency conversion service offered by
 * https://openexchangerates.org
 *
 * Further info. on this API is here:
 * https://openexchangerates.org/documentation. To work, you must first register
 * and get an application identity.
 */
public class OpenExchangeRates { // Default base currency is USD

    public static final String _License_key = "678cd96edd4b4f3eb637bf74ef8e0815";

    /**
     * Laziness: 'OpenExchangeRates.Convert' and 'OpenExchangeRates.Convert2'
     * only support USD to EUR!
     */
    public static double Convert(Currencies fromCurrency, Currencies toCurrency, double amount) throws java.net.MalformedURLException, java.io.IOException {
        if (fromCurrency == toCurrency) {
            return amount;
        }
        // Cheap access for poor people:
        java.net.URL url = new java.net.URL("http://openexchangerates.org/api/latest.json" + "?app_id=" + _License_key); // Subscription to the service is mandatory to get a license key!
        java.net.URLConnection connection = (java.net.URLConnection) url.openConnection();
        /*
         * The JSON processing API is recognized as being part of Java EE 7.
         * This situation may call for complicated installation and configuration because one may want to use JSON outside the (complex) world of Java EE.
         * To avoid this, one has to download the JSON library as self-contained component from http://mvnrepository.com/artifact/org.glassfish/javax.json.
         * The code below then supposes the access ('classpath' system variable) to the JSON library, say, 'javax.json-1.0.4.jar'
         */
        if (connection != null) {
            javax.json.stream.JsonParserFactory factory = javax.json.Json.createParserFactory(null);
            // Effective connection with the service:
            javax.json.stream.JsonParser parser = factory.createParser(connection.getInputStream());
            /**
             * This is a very rough (and probably unsafe) scanning of the
             * returned JSON object. Many improvements are possible including
             * rounding, etc.
             */
            while (parser.hasNext()) {
                javax.json.stream.JsonParser.Event event = parser.next();
                if (event == javax.json.stream.JsonParser.Event.KEY_NAME && parser.getString().equals(toCurrency.currency)) {
                    while (parser.hasNext()) {
                        event = parser.next();
                        if (event == javax.json.stream.JsonParser.Event.VALUE_NUMBER) {
                            amount *= parser.getBigDecimal().doubleValue();
                            break;
                        }
                    }
                }
            }

        }
        return amount;
    }

    public static double Convert2(Currencies fromCurrency, Currencies toCurrency, double amount) throws java.net.MalformedURLException, java.io.IOException {
        if (fromCurrency == toCurrency) {
            return amount;
        }
        java.net.URL url = new java.net.URL("http://openexchangerates.org/api/latest.json" + "?app_id=" + _License_key);
        java.net.URLConnection connection = (java.net.URLConnection) url.openConnection();
        /**
         * String-based approach (to avoid JSON)
         */
        /**
         * Currency exchange rate has a default base, which is 'USD'. For
         * simplification, 'fromCurrency' is not used
         */
        if (connection != null) {
            java.io.BufferedReader br = new java.io.BufferedReader(new java.io.InputStreamReader(connection.getInputStream()));
            String input;
            while ((input = br.readLine()) != null) {
                /**
                 * This is a very rough (and surely unsafe) scanning of the
                 * returned object. Many improvements are possible including
                 * rounding, etc.
                 */
                int index = input.lastIndexOf(toCurrency.currency);
                if (index != -1) {
                    amount *= Double.valueOf(input.substring(index + 5, input.length() - 1));
                    break;
                }
            }
            br.close();
        }
        return amount;
    }

    public static double Convert3(Currencies fromCurrency, Currencies toCurrency, double amount) throws java.net.MalformedURLException, java.io.IOException {
        if (fromCurrency == toCurrency) {
            return amount;
        }
        java.net.URL url = new java.net.URL("https://openexchangerates.org/api/convert/" + String.valueOf(amount) + "/" + fromCurrency + "/" + toCurrency + "?app_id=" + _License_key);
        javax.net.ssl.HttpsURLConnection connection = (javax.net.ssl.HttpsURLConnection) url.openConnection();
        double result = 0.;

        if (connection != null) {
            javax.json.stream.JsonParserFactory factory = javax.json.Json.createParserFactory(null);
            javax.json.stream.JsonParser parser = factory.createParser(connection.getInputStream());
            while (parser.hasNext()) {
                javax.json.stream.JsonParser.Event event = parser.next();
                /**
                 * Please extract value from JSON object below to setup
                 * 'result':
                 */
// {
//    "disclaimer": "[...]",
//    "license": "[...]",
//    "request": {
//        "query": "/convert/1000/USD/EUR",
//        "amount": 1000,
//        "from": "USD",
//        "to": "EUR"
//    },
//    "meta": {
//        "timestamp": 1371030818,
//        "rate": 0.721416,
//    },
//    "response": 721.416
// }
            }
        }
        return result;
    }
}
